#!/bin/sh

# test.sh

# Copyright Notice:

# Copyright (C) 2013 R Veera Kumar
#
# This file is part of v-tools.
#
# v-tools is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.

# v-tools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with v-tools.  If not, see <http://www.gnu.org/licenses/>.
#
# See the COPYING file distributed together for license information.
#
# See the AUTHOR file distributed together for authors information.
#

hexdump -V &> /dev/null
if [ $? -eq 0 ]; then
 HEX_PROG_EXT="hexdump"
else
 HEX_PROG_EXT=""
fi

COL=${COL:-74}

function trace ()

 {
  echo -ne "$2"
  local n=${#3}
  echo -n "$3"
  for j in `seq 1 $((${COL}-${n}))`; do echo -n ' '; done
  echo -n "$4"
  [ $1 == "-0" ] && echo ''
  [ $1 == "-n" ] && true
 }

function test1 ()
 {
  if [ -z "${HEX_PROG_EXT}" ]; then
   trace -0 '' 'Test 1: No hexdump program available ::' 'SKIP'
  else
   {
    trace -n '' 'Test 1: Byte to Hex (Small Case) ::' ''
    i=1
    i_lmt=$((1024*16))
    while [ $i -le ${i_lmt} ]; do
     local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
     dd if=/dev/urandom of="${FILE1}" bs="${i}c" count=1 &> /dev/null
     ./v-bnr -br -b16 -i "${FILE1}" -o "${FILE2}"
     ${HEX_PROG_EXT} -v -e '1/1 "%.2x"' "${FILE1}" > "${FILE3}"
     cmp "${FILE2}" "${FILE3}" &> /dev/null
     if [ $? -ne 0 ]; then
      echo "FAIL"
      trace -0 '\t' "FILE1:= ${FILE1}" ''
      trace -0 '\t' "FILE2:= ${FILE2}" ''
      trace -0 '\t' "FILE3:= ${FILE3}" ''
      trace -0 '\t' "SIZE:= ${i}" ''
      i=$(($i_lmt+1))
      false
     else
      i=$((${i}+1))
      rm -f "${FILE1}" "${FILE2}" "${FILE3}"
     fi
    done && echo "OK"
   }
  fi
 }

function test2 ()
 {
  if [ -z "${HEX_PROG_EXT}" ]; then
   trace -0 '' 'Test 2: No hexdump program available ::' 'SKIP'
  else
   {
    trace -n '' 'Test 2: Byte to Hex (Capital Case) ::' ''
    i=1
    i_lmt=$((1024*16))
    while [ $i -le ${i_lmt} ]; do
     local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
     dd if=/dev/urandom of="${FILE1}" bs="${i}c" count=1 &> /dev/null
     ./v-bnr -br -B16 -i "${FILE1}" -o "${FILE2}"
     ${HEX_PROG_EXT} -v -e '1/1 "%.2X"' "${FILE1}" > "${FILE3}"
     cmp "${FILE2}" "${FILE3}" &> /dev/null
     if [ $? -ne 0 ]; then
      echo "FAIL"
      trace -0 '\t' "FILE1:= ${FILE1}" ''
      trace -0 '\t' "FILE2:= ${FILE2}" ''
      trace -0 '\t' "FILE3:= ${FILE3}" ''
      trace -0 '\t' "SIZE:= ${i}" ''
      i=$(($i_lmt+1))
      false
     else
      i=$((${i}+1))
      rm -f "${FILE1}" "${FILE2}" "${FILE3}"
     fi
    done && echo "OK"
   }
  fi
 }


function test3 ()
 {
  if [ -z "${HEX_PROG_EXT}" ]; then
   trace -0 '' 'Test 3: No hexdump program available ::' 'SKIP'
  else
   {
    trace -n '' 'Test 3: Hex to Byte (Small Case) ::' ''
    i=1
    i_lmt=$((1024*16))
    while [ $i -le ${i_lmt} ]; do
     local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
     dd if=/dev/urandom of="${FILE1}" bs="${i}c" count=1 &> /dev/null
     ${HEX_PROG_EXT} -v -e '1/1 "%.2x"' "${FILE1}" > "${FILE2}"
     ./v-bnr -rb -b16 -i "${FILE2}" -o "${FILE3}"
     cmp "${FILE1}" "${FILE3}" &> /dev/null
     if [ $? -ne 0 ]; then
      echo "FAIL"
      trace -0 '\t' "FILE1:= ${FILE1}" ''
      trace -0 '\t' "FILE2:= ${FILE2}" ''
      trace -0 '\t' "FILE3:= ${FILE3}" ''
      trace -0 '\t' "SIZE:= ${i}" ''
      i=$(($i_lmt+1))
      false
     else
      i=$((${i}+1))
      rm -f "${FILE1}" "${FILE2}" "${FILE3}"
     fi
    done && echo "OK"
   }
  fi
 }


function test4 ()
 {
  if [ -z "${HEX_PROG_EXT}" ]; then
   trace -0 '' 'Test 4: No hexdump program available ::' 'SKIP'
  else
   {
    trace -n '' 'Test 4: Hex to Byte (Capital Case) ::' ''
    i=1
    i_lmt=$((1024*16))
    while [ $i -le ${i_lmt} ]; do
     local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
     dd if=/dev/urandom of="${FILE1}" bs="${i}c" count=1 &> /dev/null
     ${HEX_PROG_EXT} -v -e '1/1 "%.2X"' "${FILE1}" > "${FILE2}"
     ./v-bnr -rb -b16 -i "${FILE2}" -o "${FILE3}"
     cmp "${FILE1}" "${FILE3}" &> /dev/null
     if [ $? -ne 0 ]; then
      echo "FAIL"
      trace -0 '\t' "FILE1:= ${FILE1}" ''
      trace -0 '\t' "FILE2:= ${FILE2}" ''
      trace -0 '\t' "FILE3:= ${FILE3}" ''
      trace -0 '\t' "SIZE:= ${i}" ''
      i=$(($i_lmt+1))
      false
     else
      i=$((${i}+1))
      rm -f "${FILE1}" "${FILE2}" "${FILE3}"
     fi
    done && echo "OK"
   }
  fi
 }

base64 --version &> /dev/null;
if [ $? -eq 0 ]; then
 BASE64_PROG_EXT="base64"
else
 BASE64_PROG_EXT=""
fi

function test5 ()
 {
  if [ -z "${BASE64_PROG_EXT}" ]; then
   trace -0 '' 'Test 5: No base64 program available ::' 'SKIP'
  else
   {
    trace -n '' 'Test 5: Byte to Base64 ::' 
    i=1
    i_lmt=$((1024*16))
    while [ $i -le ${i_lmt} ]; do
     local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
     dd if=/dev/urandom of="${FILE1}" bs="${i}c" count=1 &> /dev/null
     ./v-bnr -br -b64 -i "${FILE1}" -o "${FILE2}"
     ${BASE64_PROG_EXT} -w 0 "${FILE1}" > "${FILE3}"
     cmp "${FILE2}" "${FILE3}" &> /dev/null
     if [ $? -ne 0 ]; then
      echo "FAIL"
      trace -0 '\t' "FILE1:= ${FILE1}" ''
      trace -0 '\t' "FILE2:= ${FILE2}" ''
      trace -0 '\t' "FILE3:= ${FILE3}" ''
      trace -0 '\t' "SIZE:= ${i}" ''
      i=$(($i_lmt+1))
      false
     else
      i=$((${i}+1))
      rm -f "${FILE1}" "${FILE2}" "${FILE3}"
     fi
    done && echo "OK"
   }
  fi
 }

function test6 ()
 {
  if [ -z "${BASE64_PROG_EXT}" ]; then
   trace -0 '' 'Test 6: No base64 program available ::' 'SKIP'
  else
   {
    trace -n '' 'Test 6: Base64 to Byte ::' 
    i=1
    i_lmt=$((1024*16))
    while [ $i -le ${i_lmt} ]; do
     local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
     local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
     dd if=/dev/urandom of="${FILE1}" bs="${i}c" count=1 &> /dev/null
     ${BASE64_PROG_EXT} -w 0 "${FILE1}" > "${FILE2}"
     ./v-bnr -rb -b64 -i "${FILE2}" -o "${FILE3}"
     cmp "${FILE1}" "${FILE3}" &> /dev/null
     if [ $? -ne 0 ]; then
      echo "FAIL"
      trace -0 '\t' "FILE1:= ${FILE1}" ''
      trace -0 '\t' "FILE2:= ${FILE2}" ''
      trace -0 '\t' "FILE3:= ${FILE3}" ''
      trace -0 '\t' "SIZE:= ${i}" ''
      i=$(($i_lmt+1))
      false
     else
      i=$((${i}+1))
      rm -f "${FILE1}" "${FILE2}" "${FILE3}"
     fi
    done && echo "OK"
   }
  fi
 }

function test7 ()
 {
  trace -n '' 'Test 7: Null to Null-Null (Incremental) ::' 
  i=1
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -si -fs "$i" -o "${FILE1}"
   ./v-bnr -ntnn -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -nntn -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test8 ()
 {
  trace -n '' 'Test 8: Null to Null-Null (Decremental) ::' 
  i=1
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -sd -fs "$i" -o "${FILE1}"
   ./v-bnr -ntnn -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -nntn -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test9 ()
 {
  trace -n '' 'Test 9: Null to Null-Null (Random) ::' 
  i=2
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -sr -fs "$i" -o "${FILE1}"
   ./v-bnr -ntnn -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -nntn -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test10 ()
 {
  trace -n '' 'Test 10: Null-Null to Null (Incremental) ::' 
  i=4
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -si -fs "$i" -o "${FILE1}"
   ./v-bnr -nntn -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -ntnn -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test11 ()
 {
  trace -n '' 'Test 11: Null-Null to Null (Decremental) ::' 
  i=9
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -sd -fs "$i" -o "${FILE1}"
   ./v-bnr -nntn -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -ntnn -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test12 ()
 {
  trace -n '' 'Test 12: Null-Null to Null (Random) ::' 
  i=4
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -sr -fs "$i" -o "${FILE1}"
   ./v-bnr -nntn -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -ntnn -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test13 ()
 {
  trace -n '' 'Test 13: Null to Base16 (Small Case) to Null (Incremental) ::' 
  i=1
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -si -fs "$i" -o "${FILE1}"
   ./v-bnr -nr -b16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test14 ()
 {
  trace -n '' 'Test 14: Null to Base16 (Small Case) to Null (Decremental) ::' 
  i=1
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -sd -fs "$i" -o "${FILE1}"
   ./v-bnr -nr -b16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test15 ()
 {
  trace -n '' 'Test 15: Null to Base16 (Small Case) to Null (Random) ::' 
  i=2
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -sr -fs "$i" -o "${FILE1}"
   ./v-bnr -nr -b16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test16 ()
 {
  trace -n '' 'Test 16: Null to Base16 (Capital Case) to Null (Incremental) ::' 
  i=1
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -si -fs "$i" -o "${FILE1}"
   ./v-bnr -nr -B16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test17 ()
 {
  trace -n '' 'Test 17: Null to Base16 (Capital Case) to Null (Decremental) ::' 
  i=1
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -sd -fs "$i" -o "${FILE1}"
   ./v-bnr -nr -B16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test18 ()
 {
  trace -n '' 'Test 18: Null to Base16 (Capital Case) to Null (Random) ::' 
  i=2
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -sr -fs "$i" -o "${FILE1}"
   ./v-bnr -nr -B16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test19 ()
 {
  trace -n '' 'Test 19: Null to Base64 to Null (Incremental) ::' 
  i=1
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -si -fs "$i" -o "${FILE1}"
   ./v-bnr -nr -b64 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rn -b64 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test20 ()
 {
  trace -n '' 'Test 20: Null to Base64 to Null (Decremental) ::' 
  i=1
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -sd -fs "$i" -o "${FILE1}"
   ./v-bnr -nr -b64 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rn -b64 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test21 ()
 {
  trace -n '' 'Test 21: Null to Base64 to Null (Random) ::' 
  i=2
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -n -sr -fs "$i" -o "${FILE1}"
   ./v-bnr -nr -b64 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rn -b64 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test22 ()
 {
  trace -n '' 'Test 22: Null-Null to Base16 (Small Case) to Null-Null (Incremental) ::' 
  i=4
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -si -fs "$i" -o "${FILE1}"
   ./v-bnr -nnr -b16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rnn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test23 ()
 {
  trace -n '' 'Test 23: Null-Null to Base16 (Small Case) to Null-Null (Decremental) ::' 
  i=9
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -sd -fs "$i" -o "${FILE1}"
   ./v-bnr -nnr -b16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rnn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test24 ()
 {
  trace -n '' 'Test 24: Null-Null to Base16 (Small Case) to Null-Null (Random) ::' 
  i=4
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -sr -fs "$i" -o "${FILE1}"
   ./v-bnr -nnr -b16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rnn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test25 ()
 {
  trace -n '' 'Test 25: Null-Null to Base16 (Capital Case) to Null-Null (Incremental) ::' 
  i=4
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -si -fs "$i" -o "${FILE1}"
   ./v-bnr -nnr -B16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rnn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test26 ()
 {
  trace -n '' 'Test 26: Null-Null to Base16 (Capital Case) to Null-Null (Decremental) ::' 
  i=9
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -sd -fs "$i" -o "${FILE1}"
   ./v-bnr -nnr -B16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rnn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test27 ()
 {
  trace -n '' 'Test 27: Null-Null to Base16 (Capital Case) to Null-Null (Random) ::' 
  i=4
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -sr -fs "$i" -o "${FILE1}"
   ./v-bnr -nnr -B16 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rnn -b16 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test28 ()
 {
  trace -n '' 'Test 28: Null-Null to Base64 to Null-Null (Incremental) ::' 
  i=4
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -si -fs "$i" -o "${FILE1}"
   ./v-bnr -nnr -b64 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rnn -b64 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test29 ()
 {
  trace -n '' 'Test 29: Null-Null to Base64 to Null-Null (Decremental) ::' 
  i=9
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -sd -fs "$i" -o "${FILE1}"
   ./v-bnr -nnr -b64 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rnn -b64 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

function test30 ()
 {
  trace -n '' 'Test 30: Null-Null to Base64 to Null-Null (Random) ::' 
  i=4
  i_lmt=$((1024*16))
  while [ $i -le ${i_lmt} ]; do
   local FILE1=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE2=`mktemp --tmpdir vtool_XXXXXXXX`
   local FILE3=`mktemp --tmpdir vtool_XXXXXXXX`
   ./test -nn -sr -fs "$i" -o "${FILE1}"
   ./v-bnr -nnr -b64 -i "${FILE1}" -o "${FILE2}"
   ./v-bnr -rnn -b64 -i "${FILE2}" -o "${FILE3}"
   cmp "${FILE1}" "${FILE3}" &> /dev/null
   if [ $? -ne 0 ]; then
    echo "FAIL"
    trace -0 '\t' "FILE1:= ${FILE1}" ''
    trace -0 '\t' "FILE2:= ${FILE2}" ''
    trace -0 '\t' "FILE3:= ${FILE3}" ''
    trace -0 '\t' "SIZE:= ${i}" ''
    i=$(($i_lmt+1))
    false
   else
    i=$((${i}+1))
    rm -f "${FILE1}" "${FILE2}" "${FILE3}"
   fi
  done && echo "OK"
 }

if [ $# -eq 1 ]; then
 { [ $1 -gt 0 ] && echo OK; } &> /dev/null
 if [ $? -eq 0 ]; then
  test$1
 else
  echo "Error: Give a valid Test Number!"
 fi
else
 for f in `seq 1 30`; do
  test$f
 done
fi

