/* v-bnr - A kind of binary/newline/radix converter for unix/linux */

// v-bnr.c

/*

Copyright Notice:

Copyright (C) 2013 R Veera Kumar

This file is part of v-tools.

v-tools is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

v-tools is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with v-tools.  If not, see <http://www.gnu.org/licenses/>.

See the COPYING file distributed together for license information.

See the AUTHOR file distributed together for authors information.

*/

/* USAGE: v-bnr -nnr | -nr | -rnn | -rn | -nntn | -ntnn | -br | -rb | -nnlnn | [ -b16 | -B16 | -b64 ] | -i FILE | -o FILE | -h | -V */

#include "v-bnr.h"

FILE *fpin;
FILE *fpout;
uint8_t *mem_bufa, *mem_bufb;

int main ( int argc, char **argv )
 {

  extern FILE *fpin;
  extern FILE *fpout;
  fpin = stdin;
  fpout = stdout;
  int arg_i, arg_o, nnr, nr, rnn, rn, nntn, ntnn, br, rb, nnlnn, b16, b64;
  arg_i = arg_o = nnr = nr = rnn = rn = nntn = ntnn = br = rb = nnlnn = b16 = b64 = 0;
  int stat, cnt;
  char *ifn, *ofn;
  extern uint8_t *mem_bufa, *mem_bufb;
  size_t read_sum, proc_rem, write_sum, i, j, str_len, mem_bufa_sum, mem_bufb_sum;

  if ( argc > 7 )
   { usage(); exit(1); }


  cnt = 1;
  while ( cnt <= (argc-1) )
   {

    if ( strcmp(argv[cnt], "-i") == 0 )
     {
      if ( arg_i != 1 )
       { arg_i = 1; if ( (cnt+1) > (argc-1) ) { opt_arg_err(); exit(1); }; ifn = argv[cnt+1]; cnt +=2; continue; }
      else { opt_err(); usage(); exit(1); }
     }

    else if ( strcmp(argv[cnt], "-o") == 0 )
     {
      if ( arg_o != 1 )
       { arg_o = 1; if ( (cnt+1) > (argc-1) ) { opt_arg_err(); exit(1); }; ofn = argv[cnt+1]; cnt +=2; continue; }
      else { opt_err(); usage(); exit(1); }
     }

    else if ( strcmp(argv[cnt], "-nr") == 0 ) { nr = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-nnr") == 0 ) { nnr = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-rn") == 0 ) { rn = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-rnn") == 0 ) { rnn = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-ntnn") == 0 ) { ntnn = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-nntn") == 0 ) { nntn = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-br") == 0 ) { br = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-rb") == 0 ) { rb = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-nnlnn") == 0 ) { nnlnn = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-b16") == 0 ) { b16 = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-B16") == 0 ) { b16 = 2; ++cnt; }
    else if ( strcmp(argv[cnt], "-b64") == 0 ) { b64 = 1; ++cnt; }
    else if ( strcmp(argv[cnt], "-h") == 0 ) { usage(); exit(0); }
    else if ( strcmp(argv[cnt], "-V") == 0 ) { version(); exit(0); }
    else { parm_unknown_err(argv[cnt]); usage(); exit(1); }

   }

  if ( ( (nr == 1) || (nnr == 1) || (rn == 1) || (rnn == 1) || (br == 1) || (rb == 1) ) )
   {
    if ( (b16 > 0) && (b64 == 1) )
     {
      fputs("You can give one radix only. Either -b16 or -B16 or -b64! Exiting...\n", stderr);
      exit(1);
     }
    else if ( (b16 == 0) && (b64 != 1) )
     {
     fputs("You have to specify the radix. Either -b16 or -B16 or -b64! Exiting...\n", stderr);
     exit(1);
     }
   }

  if ( arg_i == 1 )
   {
    if ( (fpin = fopen(ifn, "r")) == NULL ) { file_open_err(); exit(1); }
   }

  if ( arg_o == 1 )
   {
    if ( (fpout = fopen(ofn, "w")) == NULL ) { file_open_err(); exit(1); }
   }


  atexit(exit_bye);

  /* -nr */

   /* -b16 */

  if ( (nr == 1) && (b16 > 0) )
   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE * 2)) == NULL ) { mem_alloc_err(); exit(1); }

    do
     {

      read_sum = readtobuf ( mem_bufa, BUFSIZE, fpin );

      if ( read_sum > 0 )

       {

        mem_bufa_sum = mem_bufb_sum = i = 0;

        do
         {  
          while ( (i < read_sum) && ( *(mem_bufa+i) != 0 ) ) ++i;

          str_len = i - mem_bufa_sum;

          if (str_len) mem_bufb_sum += br16 ( mem_bufa + mem_bufa_sum, mem_bufb + mem_bufb_sum, str_len, b16 );

          if ( (i != read_sum) && *(mem_bufa+i) == 0 ) 
           {
            *(mem_bufb + mem_bufb_sum) = '\n';
            ++mem_bufb_sum;

            ++i;
            mem_bufa_sum = i;
           }

         }
        while ( i < read_sum );

        write_sum = mem_bufb_sum;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  /* -nr */

   /* -b64 */

  if ( (nr == 1) && ( b64 == 1 ) )
   {

    if ( (mem_bufa = malloc(BUFSIZE_R64_3)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE_R64_4 * 2)) == NULL ) { mem_alloc_err(); exit(1); }

    proc_rem = 0;

    do
     {

      read_sum = readtobuf ( mem_bufa + proc_rem, BUFSIZE_R64_3 - proc_rem, fpin ) + proc_rem;
      proc_rem = 0;

      if ( read_sum > 0 )

       {

        mem_bufa_sum = mem_bufb_sum = i = 0;

        do
         {  
          while ( (i < read_sum) && ( *(mem_bufa+i) != 0 ) ) ++i;

          if (i == read_sum)
           {
            proc_rem = (i - mem_bufa_sum) % 3;
            str_len = i - mem_bufa_sum - proc_rem;
           }
          else str_len = i - mem_bufa_sum;

          if (str_len) mem_bufb_sum += br64 ( mem_bufa + mem_bufa_sum, mem_bufb + mem_bufb_sum, str_len );

          if ( (i != read_sum) && *(mem_bufa+i) == 0 ) 
           {
            *(mem_bufb + mem_bufb_sum) = '\n';
            ++mem_bufb_sum;

            ++i;
            mem_bufa_sum = i;
           }

          if (i == read_sum)
           {
            if ( proc_rem == 1 ) { *(mem_bufa) = *(mem_bufa+read_sum-1); }
            else if ( proc_rem == 2 )
             {
              *(mem_bufa) = *(mem_bufa+read_sum-2);
              *(mem_bufa+1) = *(mem_bufa+read_sum-1);
             }
           }
         }
        while ( i < read_sum );

        write_sum = mem_bufb_sum;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE_R64_3);

    exit(0);

   }

  /* -ntnn */

  if ( ntnn == 1 )
   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE * 2)) == NULL ) { mem_alloc_err(); exit(1); }

    stat = 0;

    do
     {

      read_sum = readtobuf ( mem_bufa, BUFSIZE, fpin );

      if ( read_sum > 0 )

       {

        i = 0;
        j = 0;
        while ( i < read_sum )
         {
          if ( stat == 0 )
           {
            *(mem_bufb+j++) = '\0';
            stat = 1;
            continue;
           }
          else if ( *(mem_bufa+i) != 0 )
            *(mem_bufb+j++) = *(mem_bufa+i++);
          else if ( stat == 1 )
           {
            *(mem_bufb+j++) = '\0';
            stat = 0;
            ++i;
           }
         }

        write_sum = j;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  /* -nnr */

   /* -b16 */

  if ( (nnr == 1) && (b16 > 0) )
   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE * 2)) == NULL ) { mem_alloc_err(); exit(1); }

    stat = 0;

    do
     {

      read_sum = readtobuf ( mem_bufa, BUFSIZE, fpin );

      if ( read_sum > 0 )

       {

        mem_bufa_sum = mem_bufb_sum = i = 0;
        
        do
         {

          if ( stat == 0 ) { stat = 1; ++i; ++mem_bufa_sum; continue; }
           
          while ( (i < read_sum) && ( *(mem_bufa+i) != 0 ) ) ++i;

          str_len = i - mem_bufa_sum;

          if ( str_len ) mem_bufb_sum += br16 ( mem_bufa + mem_bufa_sum, mem_bufb + mem_bufb_sum, str_len, b16 );

          if ( (i != read_sum) && *(mem_bufa+i) == 0 ) 
           {
            *(mem_bufb + mem_bufb_sum) = '\n';
            ++mem_bufb_sum;

            ++i;
            mem_bufa_sum = i;
            stat = 0;
           }

         }
        while ( i < read_sum );

        write_sum = mem_bufb_sum;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  /* -nnr */

   /* -b64 */

  if ( (nnr == 1) && (b64 == 1) )
   {

    if ( (mem_bufa = malloc(BUFSIZE_R64_3)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE_R64_4 * 2)) == NULL ) { mem_alloc_err(); exit(1); }

    proc_rem = 0;
    stat = 0;

    do
     {

      read_sum = readtobuf ( mem_bufa + proc_rem, BUFSIZE_R64_3 - proc_rem, fpin ) + proc_rem;
      proc_rem = 0;

      if ( read_sum > 0 )

       {

        mem_bufa_sum = mem_bufb_sum = i = 0;
        
        do
         {

          if ( stat == 0 ) { stat = 1; ++i; ++mem_bufa_sum; continue; }
           
          while ( (i < read_sum) && ( *(mem_bufa+i) != 0 ) ) ++i;

          if (i == read_sum)
           {
            proc_rem = (i - mem_bufa_sum) % 3;
            str_len = i - mem_bufa_sum - proc_rem;
           }
          else str_len = i - mem_bufa_sum;

          if ( str_len ) mem_bufb_sum += br64 ( mem_bufa + mem_bufa_sum, mem_bufb + mem_bufb_sum, str_len );

          if ( (i != read_sum) && *(mem_bufa+i) == 0 ) 
           {
            *(mem_bufb + mem_bufb_sum) = '\n';
            ++mem_bufb_sum;

            ++i;
            mem_bufa_sum = i;
            stat = 0;
           }

          if (i == read_sum)
           {
            if ( proc_rem == 1 ) { *(mem_bufa) = *(mem_bufa+read_sum-1); }
            else if ( proc_rem == 2 )
             {
              *(mem_bufa) = *(mem_bufa+read_sum-2);
              *(mem_bufa+1) = *(mem_bufa+read_sum-1);
             }
           }

         }
        while ( i < read_sum );

        write_sum = mem_bufb_sum;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE_R64_3);

    exit(0);

   }

  /* -nntn */

  if ( nntn == 1 )
   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }

    stat = 0;

    do
     {

      read_sum = readtobuf ( mem_bufa, BUFSIZE, fpin );

      if ( read_sum > 0 )

       {

        for( i = 0, j = 0; i < read_sum; ++i )
         {
          if ( *(mem_bufa+i) != 0 ) *(mem_bufb+j++) = *(mem_bufa+i);
          else if ( stat == 0 )
           {
            stat = 1;
            continue;
           }
          else if ( stat == 1 )
           {
            *(mem_bufb+j++) = '\0';
            stat = 0;
           }
         }

        write_sum = j;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  /* -rn */

   /* -b16 */

  if ( (rn == 1) && ( b16 > 0) )
   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }

    proc_rem = 0;

    do
     {

      read_sum = readtobuf ( mem_bufa + proc_rem, BUFSIZE - proc_rem, fpin ) + proc_rem;
      proc_rem = 0;

      if ( read_sum > 0 )

       {

        mem_bufa_sum = mem_bufb_sum = i = 0;

        do
         {  
          while ( (i < read_sum) && ( *(mem_bufa+i) != '\n' ) ) ++i;

          if (i == read_sum)
           {
            proc_rem = (i - mem_bufa_sum) % 2;
            str_len = i - mem_bufa_sum - proc_rem;
           }
          else str_len = i - mem_bufa_sum;

          if (str_len) mem_bufb_sum += r16b ( mem_bufa + mem_bufa_sum, mem_bufb + mem_bufb_sum, str_len );

          if ( (i != read_sum) && *(mem_bufa+i) == '\n' ) 
           {
            *(mem_bufb + mem_bufb_sum) = '\0';
            ++mem_bufb_sum;

            ++i;
            mem_bufa_sum = i;
           }

          if (i == read_sum)
           {
            if ( proc_rem == 1 ) { *(mem_bufa) = *(mem_bufa+read_sum-1); }
           }

         }
        while ( i < read_sum );

        write_sum = mem_bufb_sum;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  /* -rn */

   /* -b64 */

  if ( (rn == 1) && ( b64 == 1) )
   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }

    proc_rem = 0;

    do
     {

      read_sum = readtobuf ( mem_bufa + proc_rem, BUFSIZE - proc_rem, fpin ) + proc_rem;
      proc_rem = 0;

      if ( read_sum > 0 )

       {

        mem_bufa_sum = mem_bufb_sum = i = 0;

        do
         {  
          while ( (i < read_sum) && ( *(mem_bufa+i) != '\n' ) ) ++i;

          if (i == read_sum)
           {
            proc_rem = (i - mem_bufa_sum) % 4;
            str_len = i - mem_bufa_sum - proc_rem;
           }
          else str_len = i - mem_bufa_sum;

          if (str_len) mem_bufb_sum += r64b ( mem_bufa + mem_bufa_sum, mem_bufb + mem_bufb_sum, str_len );

          if ( (i != read_sum) && *(mem_bufa+i) == '\n' ) 
           {
            *(mem_bufb + mem_bufb_sum) = '\0';
            ++mem_bufb_sum;

            ++i;
            mem_bufa_sum = i;
           }

          if (i == read_sum)
           {
            if ( proc_rem == 1 ) { *(mem_bufa) = *(mem_bufa+read_sum-1); }
            else if ( proc_rem == 2 )
             {
              *(mem_bufa) = *(mem_bufa+read_sum-2);
              *(mem_bufa+1) = *(mem_bufa+read_sum-1);
             }
            else if ( proc_rem == 3 )
             {
              *(mem_bufa) = *(mem_bufa+read_sum-3);
              *(mem_bufa+1) = *(mem_bufa+read_sum-2);
              *(mem_bufa+2) = *(mem_bufa+read_sum-1);
             }
           }

         }
        while ( i < read_sum );

        write_sum = mem_bufb_sum;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  /* -rnn */

   /* -b16 */

  if ( (rnn == 1) && ( b16 > 0) )
   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE * 2)) == NULL ) { mem_alloc_err(); exit(1); }

    proc_rem = 0;
    stat = 0;

    do
     {

      read_sum = readtobuf ( mem_bufa + proc_rem, BUFSIZE - proc_rem, fpin ) + proc_rem;
      proc_rem = 0;

      if ( read_sum > 0 )

       {

        mem_bufa_sum = mem_bufb_sum = i = 0;

        do
         {  

          if ( stat == 0 )
           {
            *(mem_bufb + mem_bufb_sum) = '\0';
            ++mem_bufb_sum;
            stat = 1;
           }

          while ( (i < read_sum) && ( *(mem_bufa+i) != '\n' ) ) ++i;

          if (i == read_sum)
           {
            proc_rem = (i - mem_bufa_sum) % 2;
            str_len = i - mem_bufa_sum - proc_rem;
           }
          else str_len = i - mem_bufa_sum;

          if (str_len) mem_bufb_sum += r16b ( mem_bufa + mem_bufa_sum, mem_bufb + mem_bufb_sum, str_len );

          if ( (i != read_sum) && *(mem_bufa+i) == '\n' ) 
           {
            *(mem_bufb + mem_bufb_sum) = '\0';
            ++mem_bufb_sum;

            ++i;
            mem_bufa_sum = i;
            stat = 0;
           }

          if (i == read_sum)
           {
            if ( proc_rem == 1 ) { *(mem_bufa) = *(mem_bufa+read_sum-1); }
           }

         }
        while ( i < read_sum );

        write_sum = mem_bufb_sum;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
         { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  /* -rnn */

   /* -b64 */

  if ( (rnn == 1) && ( b64 == 1) )
   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE * 2)) == NULL ) { mem_alloc_err(); exit(1); }

    proc_rem = 0;
    stat = 0;

    do
     {

      read_sum = readtobuf ( mem_bufa + proc_rem, BUFSIZE - proc_rem, fpin ) + proc_rem;
      proc_rem = 0;

      if ( read_sum > 0 )

       {

        mem_bufa_sum = mem_bufb_sum = i = 0;

        do
         {  

          if ( stat == 0 )
           {
            *(mem_bufb + mem_bufb_sum) = '\0';
            ++mem_bufb_sum;
            stat = 1;
           }

          while ( (i < read_sum) && ( *(mem_bufa+i) != '\n' ) ) ++i;

          if (i == read_sum)
           {
            proc_rem = (i - mem_bufa_sum) % 4;
            str_len = i - mem_bufa_sum - proc_rem;
           }
          else str_len = i - mem_bufa_sum;

          if (str_len) mem_bufb_sum += r64b ( mem_bufa + mem_bufa_sum, mem_bufb + mem_bufb_sum, str_len );

          if ( (i != read_sum) && *(mem_bufa+i) == '\n' ) 
           {
            *(mem_bufb + mem_bufb_sum) = '\0';
            ++mem_bufb_sum;

            ++i;
            mem_bufa_sum = i;
            stat = 0;
           }

          if (i == read_sum)
           {
            if ( proc_rem == 1 ) { *(mem_bufa) = *(mem_bufa+read_sum-1); }
            else if ( proc_rem == 2 )
             {
              *(mem_bufa) = *(mem_bufa+read_sum-2);
              *(mem_bufa+1) = *(mem_bufa+read_sum-1);
             }
            else if ( proc_rem == 3 )
             {
              *(mem_bufa) = *(mem_bufa+read_sum-3);
              *(mem_bufa+1) = *(mem_bufa+read_sum-2);
              *(mem_bufa+2) = *(mem_bufa+read_sum-1);
             }
           }

         }
        while ( i < read_sum );

        write_sum = mem_bufb_sum;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  /* -br */

   /* -b16 */

  if ( (br == 1) && (b16 > 0) )

   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE * 2)) == NULL ) { mem_alloc_err(); exit(1); }

    do
     {

      read_sum = readtobuf ( mem_bufa, BUFSIZE, fpin );

      if ( read_sum > 0 )

       {

        write_sum = br16 ( mem_bufa, mem_bufb, read_sum, b16 );

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  /* -br */

   /* -b64 */

  if ( (br == 1) && (b64 == 1) )

   {

    if ( (mem_bufa = malloc(BUFSIZE_R64_3)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE_R64_4)) == NULL ) { mem_alloc_err(); exit(1); }

    do
     {

      read_sum = readtobuf ( mem_bufa, BUFSIZE_R64_3, fpin );

      if ( read_sum > 0 )

       {

        write_sum = br64 ( mem_bufa, mem_bufb, read_sum );

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE_R64_3);

    exit(0);

   }


  /* -rb */

   /* -b16 */

  if ( (rb == 1) && (b16 > 0) )

   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE / 2)) == NULL ) { mem_alloc_err(); exit(1); }

    do
     {

      read_sum = readtobuf ( mem_bufa, BUFSIZE, fpin );

      if ( read_sum > 0 )

       {

        write_sum = r16b ( mem_bufa, mem_bufb, read_sum );

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
         { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  /* -rb */

   /* -b64 */

  if ( (rb == 1) && (b64 == 1) )

   {

    if ( (mem_bufa = malloc(BUFSIZE_R64_4)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE_R64_3)) == NULL ) { mem_alloc_err(); exit(1); }

    do
     {

      read_sum = readtobuf ( mem_bufa, BUFSIZE_R64_4, fpin );

      if ( read_sum > 0 )

       {

        write_sum = r64b ( mem_bufa, mem_bufb, read_sum );

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
         { file_write_err(); exit(1); }

        if ( ( read_sum % 4 ) != 0 )
         { fputs("!Error: Input file length not multiple of 4\n", stderr); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE_R64_4);

    exit(0);

   }


/* -nnlnn */

  if ( nnlnn == 1 )
   {

    if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
    if ( (mem_bufb = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }

    stat = 0;

    do
     {

      read_sum = readtobuf ( mem_bufa, BUFSIZE, fpin );

      if ( read_sum > 0 )

       {

        for( i = 0, j = 0; i < read_sum; ++i )
         {
          if ( *(mem_bufa+i) != 0 ) *(mem_bufb+j++) = *(mem_bufa+i);
          else if ( stat == 0 )
           {
            *(mem_bufb+j++) = '\0';
            stat = 1;
           }
          else if ( stat == 1 )
           {
            *(mem_bufb+j++) = '\0';
            stat = 0;
            ++i;
           }
         }

        write_sum = j;

        if (fwrite(mem_bufb, sizeof(uint8_t), write_sum, fpout) != write_sum)
        { file_write_err(); exit(1); }

       }

     }
    while (!feof (fpin) && read_sum == BUFSIZE);

    exit(0);

   }

  return 0;

 }

size_t br16 ( uint8_t *mem_bufa, uint8_t *mem_bufb, size_t read_sum, int flag )

 {

  size_t i;
  uint8_t j;
  
  for(i = 0; i < read_sum; ++i)
   {
    j = *(mem_bufa+i)/16;
    j = ( j <= 9 ? j + '0' : j - 10 + ( flag == 1 ? 'a' : 'A') );
    *(mem_bufb+(i*2)) = j;
    j = *(mem_bufa+i)%16;
    j = ( j <= 9 ? j + '0' : j - 10 + ( flag == 1 ? 'a' : 'A') );
    *(mem_bufb+(i*2)+1) = j;
   }

  return i * 2;

 }

size_t r16b ( uint8_t *mem_bufa, uint8_t *mem_bufb, size_t read_sum )

 {

  size_t i;
  uint8_t j, k;
  
  for(i = 0; i < read_sum; ++i, ++i)
   {
    j = *(mem_bufa+i);
    k = ( j <= '9' ? j - '0' : ( j <= 'Z' ? j - 'A' : j - 'a' ) + 10 ) * 16;
    j = *(mem_bufa+i+1);
    j = ( j <= '9' ? j - '0' : ( j <= 'Z' ? j - 'A' : j - 'a' ) + 10 );
    k += j;
    *(mem_bufb+(i/2)) = k;
   }

  return i / 2;

 }

size_t br64 ( uint8_t *mem_bufa, uint8_t *mem_bufb, size_t read_sum )

 {

  extern const uint8_t b64enc_tbl[]; 

  size_t oct3_num, i, j;

  oct3_num = read_sum - (read_sum % 3);

  for ( i = 0, j = 0; i < oct3_num; i += 3 )
   {
    *(mem_bufb+j++) = b64enc_tbl[ *(mem_bufa+i) >> 2];
    *(mem_bufb+j++) = b64enc_tbl[ ( (*(mem_bufa+i+1) & 0xf0) >> 4) | ( (*(mem_bufa+i) & 0x3) << 4 ) ];
    *(mem_bufb+j++) = b64enc_tbl[ ( (*(mem_bufa+i+1) & 0xf) << 2) | ( ( *(mem_bufa+i+2) & 0xc0) >> 6)  ];
    *(mem_bufb+j++) = b64enc_tbl[ *(mem_bufa+i+2) & 0x3f ];
   }

  if ( (read_sum % 3) == 2 )
   {
     *(mem_bufb+j++) = b64enc_tbl[*(mem_bufa+i) >> 2 ];
     *(mem_bufb+j++) = b64enc_tbl[ ( (*(mem_bufa+i+1) & 0xf0 ) >> 4 ) | ( ( *(mem_bufa+i) & 0x3 ) << 4 ) ];
     *(mem_bufb+j++) = b64enc_tbl[ ( ( *(mem_bufa+i+1) & 0xf ) << 2 ) ];
     *(mem_bufb+j++) = '=';
   }
  else if ( (read_sum % 3) == 1 )
   {
    *(mem_bufb+j++) = b64enc_tbl[ *(mem_bufa+i) >> 2 ];
    *(mem_bufb+j++) = b64enc_tbl[ ( ( *(mem_bufa+i) & 0x3 ) << 4 ) ];
    *(mem_bufb+j++) = '=';
    *(mem_bufb+j++) = '=';
   }

  return j;

 }

size_t r64b ( uint8_t *mem_bufa, uint8_t *mem_bufb, size_t read_sum )

 {

  extern const uint8_t b64dec_tbl[];

  size_t i, j;

  for ( i = 0, j = 0; i < read_sum; i += 4 )
   {
    *(mem_bufb+j++) = ( ( b64dec_tbl[*(mem_bufa+i)] << 2 ) | ( b64dec_tbl[*(mem_bufa+i+1)] >> 4 ) );

    if ( *(mem_bufa+i+2) != '=' )
     *(mem_bufb+j++) = (
                        ( b64dec_tbl[*(mem_bufa+i+1)] << 4 ) |
                        ( ( b64dec_tbl[*(mem_bufa+i+2)] >> 2 ) & 0xf )
                       );

    if ( *(mem_bufa+i+3) != '=' )    
     *(mem_bufb+j++) = (
                        ( ( b64dec_tbl[*(mem_bufa+i+2)] & 0x3 ) << 6 )|
                        ( b64dec_tbl[*(mem_bufa+i+3)] )
                       );
   }

  return j;

 }

size_t readtobuf ( uint8_t *mem_buf, size_t buflen, FILE *fpin )

 {

  size_t read_sum, read_cnt;

  read_sum = 0;

  do
   {
    read_cnt = fread(mem_buf + read_sum, sizeof(uint8_t), buflen - read_sum, fpin);
    read_sum += read_cnt;
   }
  while (!feof (fpin) && !ferror (fpin) && read_sum < buflen);

  if(ferror(fpin))
   { file_read_err(); exit(1); }

  return read_sum;

 }

void exit_bye ( void )

 {

  extern FILE *fpin;
  extern FILE *fpout;
  extern uint8_t *mem_bufa, *mem_bufb;

  free(mem_bufa);
  free(mem_bufb);
  fclose(fpin);
  fclose(fpout);

 }

void usage (void)
 {
  fputs(DESC "\n\n", stderr);
  fputs("Usage:\n", stderr);
  fputs("\tv-bnr -nr | -nnr | -rn | -rnn | -ntnn | -nntn | -nnlnn | -br | -rb\n", stderr);
  fputs("\t | [ -b16 | -B16 | -b64 ] | [ -i FILE ] | [ -o FILE ] | -h | -V\n", stderr);
  fputs("\n\tEach argument can be given only once.\n", stderr);
  fputs("\t-i\t Input file. If not given takes input from stdin.\n", stderr);
  fputs("\t-o\t Output file; Truncating if exists. If not given outputs to stdout.\n\n", stderr);
  fputs("\t-nr\t Change null separated bytes to radix separated by newline.\n", stderr);
  fputs("\t-nnr\t Change double null enclosed bytes to radix separated by newline.\n", stderr);
  fputs("\t-rn\t Change radix separated by newline to null separated bytes.\n", stderr);
  fputs("\t-rnn\t Change radix separated by newline to double null enclosed bytes.\n", stderr);
  fputs("\t-ntnn\t Change null separated bytes to double null enclosed bytes.\n", stderr);
  fputs("\t-nntn\t Change double null enclosed bytes to null separated bytes.\n", stderr);
  fputs("\t-nnlnn\t Change double null enclosed bytes with trailing linefeed to\n", stderr);
  fputs("\t\t double null enclosed bytes.\n", stderr);
  fputs("\t-br\t Change bytes to radix.\n", stderr);
  fputs("\t-rb\t Change radix to bytes.\n", stderr);
  fputs("\t-b16\t Radix is base16. Encode to small case.\n", stderr);
  fputs("\t-B16\t Radix is base16. Encode to capital case.\n", stderr);
  fputs("\t\t Either of -b16 or -B16 can given to decode. Also decodes mixed.\n", stderr);
  fputs("\t-b64\t Radix is base64 (RFC 4648).\n", stderr);
  fputs("\t-h\t Show help.\n", stderr);
  fputs("\t-V\t Show License and Version.\n", stderr);
 }

void version (void)
 {
  fputs(DESC"\n", stderr);
  fputs("Version: " VERSION "\n", stderr);
  fputs("License: " LICENSE "\n", stderr);
 }

void opt_err (void)
 {
  fputs("Each argument can be given only once!\n", stderr);
 }

void opt_arg_err (void)
 {
  fputs("Neccessary argument for specified option not given!\n", stderr);
 }

void file_open_err (void)
 {
  fputs("Could not open file for read/write!\n", stderr);
 }

void parm_unknown_err (char *arg)
 {
  fprintf(stderr, "Unknown argument given!: %s\n", arg);
 }

void file_write_err (void)
 {
  fputs("Write Error! Exiting...\n", stderr);
 }

void file_read_err (void)
 {
  fputs("Read Error! Exiting...\n", stderr);
 }

void mem_alloc_err (void)
 {
  fputs("Error: Unable to Get Memory! Exiting...\n", stderr);
 }

void snprintf_overflow_err (void)
 {
  fputs("Error: snprintf outputted more than required size! Exiting...\n", stderr);
 }

