/* v-bnr.h */

/*

Copyright Notice:

Copyright (C) 2013 R Veera Kumar

This file is part of v-tools.

v-tools is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

v-tools is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with v-tools.  If not, see <http://www.gnu.org/licenses/>.

See the COPYING file distributed together for license information.

See the AUTHOR file distributed together for authors information.

*/

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include <string.h>

void exit_bye ( void );
void usage (void);
void version (void);

size_t  br16 ( uint8_t *mem_bufa, uint8_t *mem_bufb, size_t read_sum, int flag );

size_t  r16b ( uint8_t *mem_bufa, uint8_t *mem_bufb, size_t read_sum );

size_t br64 ( uint8_t *mem_bufa, uint8_t *mem_bufb, size_t read_sum );

size_t r64b ( uint8_t *mem_bufa, uint8_t *mem_bufb, size_t read_sum );

size_t nnr16 ( uint8_t *mem_bufa, uint8_t *mem_bufb, size_t read_sum );

size_t readtobuf ( uint8_t *mem_buf, size_t buflen, FILE *fpin );

void usage (void);

void version (void);

void opt_err (void);

void opt_arg_err (void);

void file_open_err (void);

void parm_unknown_err (char *arg);

void file_write_err (void);

void file_read_err (void);

void mem_alloc_err (void);

void snprintf_overflow_err (void);

#define DESC "Bnr - A kind of binary/newline/radix converter"
#define VERSION "0.2"
#define LICENSE "GNU GPL Version 3"

#define BUFSIZE 4096

#if BUFSIZE % 2 != 0
# error "Invalid BUFSIZE! Should be a multiple of 2 "
#endif


/* BUFSIZE_R64_3:BUFSIZE_R64_4::3:4 ratio to be maintained */
#define BUFSIZE_R64_3 3072

#define BUFSIZE_R64_4 4096

// Ensure ratio maintained
#if BUFSIZE_R64_4 / 4 * 3 != BUFSIZE_R64_3
# error "Invalid ratio of BUFSIZE_R64_3:BUFSIZE_R64_4"
#endif


static const uint8_t b64enc_tbl[64] =
 {
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 
  'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
  'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
  'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/'
 };


static const uint8_t b64dec_tbl[128] =
 {
// Each element has been given value serially with key positions
// replaced with required values, for easy checking
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15,
  16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
  32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 62, 44, 45, 46, 63,
  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 58, 59, 60, 61, 62, 63,
  64, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 91, 92, 93, 94, 95,
  96, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 123, 124, 125, 126, 127
 };

