/* v-filestz -  ASCII TEXT Sanitizer

    Only ASCII letters from (8-13) and (32 - 126) are there in the output

*/

// v-filestz.c

/*

Copyright Notice:

Copyright (C) 2013 R Veera Kumar

This file is part of v-tools.

v-tools is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

v-tools is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with v-tools.  If not, see <http://www.gnu.org/licenses/>.

See the COPYING file distributed together for license information.

See the AUTHOR file distributed together for authors information.

*/

/* USAGE: v-filestz [ -h ] | [ -V ] | [ -i FILE ] | [ -o FILE ] */

#include "v-filestz.h"

int main ( int argc, char *argv[] )

 {

  FILE *fpin = stdin;
  FILE *fpout = stdout;
  int cnt, i, o;
  char *ifn, *ofn;
  uint8_t *mem_bufa, *mem_bufb;
  size_t read_sum, ascii_cnt;

  if ( ( argc > 5 ) )
   { usage(); exit(1); }

  i = o = 0;

  if ( (mem_bufa = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }
  if ( (mem_bufb = malloc(BUFSIZE)) == NULL ) { mem_alloc_err(); exit(1); }


  cnt = 1;
  while ( cnt <= (argc-1) )
   {

    if ( strcmp(argv[cnt], "-i") == 0 )
     if ( i != 1 )
      { i = 1; if ( (cnt+1) > (argc-1) ) { opt_arg_err(); exit(1); }; ifn = argv[cnt+1]; cnt +=2; continue; }
     else { opt_err(); usage(); exit(1); }

    else if ( strcmp(argv[cnt], "-o") == 0 )
     if ( o != 1 )
      { o = 1; if ( (cnt+1) > (argc-1) ) { opt_arg_err(); exit(1); }; ofn = argv[cnt+1]; cnt +=2; continue; }
     else { opt_err(); usage(); exit(1); }

    else if ( strcmp(argv[cnt], "-h") == 0 )
     { usage(); exit(0); }

    else if ( strcmp(argv[cnt], "-V") == 0 )
     { version(); exit(0); }

    else { parm_unknown_err(argv[cnt]); usage(); exit(1); }

   }

  if ( i == 1 )
   {
    if ( (fpin = fopen(ifn, "r")) == NULL ) { file_open_err(); exit(1); }
   }

  if ( o == 1 )
   {
    if ( (fpout = fopen(ofn, "w")) == NULL ) { file_open_err(); exit(1); }
   }

  do
   {

    read_sum = readtobuf ( mem_bufa, BUFSIZE, fpin );

    if ( read_sum > 0 )

     {

      ascii_cnt = ascii_stz ( mem_bufa, mem_bufb, read_sum );

      if (fwrite(mem_bufb, sizeof(uint8_t), ascii_cnt, fpout) != ascii_cnt )
       { file_write_err(); exit(1); }

     }

   }
  while (!feof (fpin) && read_sum == BUFSIZE);


  fclose(fpin);
  fclose(fpout);

  free(mem_bufa);
  free(mem_bufb);

  return 0;

 }

size_t ascii_stz ( uint8_t *data_in, uint8_t *data_out, size_t inlen )

 {

  size_t i, j;

  for( i = 0, j = 0; i < inlen; ++i)
   {
    if ( (*(data_in+i) < 128) && ( ( (*(data_in+i) > 31) && (*(data_in+i) < 127) ) || ( ( *(data_in+i) > 7) && ( *(data_in+i) < 14) ) ) )
     { *(data_out+j) = *(data_in+i); ++j; }
   }

  return j;

 }

size_t readtobuf ( uint8_t *mem_buf, size_t buflen, FILE *fpin )

 {

  size_t read_sum, read_cnt;

  read_sum = 0;

  do
   {
    read_cnt = fread(mem_buf + read_sum, sizeof(uint8_t), buflen - read_sum, fpin);
    read_sum += read_cnt;
   }
  while (!feof (fpin) && !ferror (fpin) && read_sum < buflen);

  if(ferror(fpin))
   { file_read_err(); exit(1); }

  return read_sum;

 }

void usage ( void )

 {
 
   fputs(DESC "\n", stderr);
   fputs("Only letters from (8-13) and (32-126) are kept.\n\n", stderr);
   fputs("Usage:\n", stderr);
   fputs("\tv-filestz  [ -h ] | [ -V ] | [ -i FILE ] | [ -o FILE ]\n", stderr);
   fputs("\n\tEach argument can be given only once.\n\n", stderr);
   fputs("\t-i\t Input file. If not given takes input from stdin.\n", stderr);
   fputs("\t-o\t Output file; Truncating if exists. If not given outputs to stdout.\n", stderr);
   fputs("\t-h\t Show usage.\n", stderr);
   fputs("\t-V\t Show License and Version.\n", stderr);

 }

void version (void)

 {

  fputs(DESC "\n", stderr);
  fputs("Version: " VERSION "\n", stderr);
  fputs("License: " LICENSE "\n", stderr);

 }

void opt_err (void)
 {
  fputs("Each argument can be given only once!\n", stderr);
 }

void opt_arg_err (void)
 {
  fputs("Neccessary argument for specified option not given!\n", stderr);
 }

void file_open_err (void)
 {
  fputs("Could not open file for read/write!\n", stderr);
 }

void parm_unknown_err (char *arg)
 {
  fprintf(stderr, "Unknown argument given!: %s\n", arg);
 }

void file_write_err (void)
 {
  fputs("Write Error! Exiting...\n", stderr);
 }

void file_read_err (void)
 {
  fputs("Read Error! Exiting...\n", stderr);
 }

void mem_alloc_err (void)
 {
  fputs("Error: Unable to Get Memory! Exiting...\n", stderr);
 }

