// v-reboot - Execute a Linux reboot syscall

// v-reboot.c

/*

Copyright Notice:

Copyright (C) 2013 R Veera Kumar

This file is part of v-tools.

v-tools is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

v-tools is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with v-tools.  If not, see <http://www.gnu.org/licenses/>.

See the COPYING file distributed together for license information.

See the AUTHOR file distributed together for authors information.

*/

/*

USAGE: v-rebootp -h | -V | cad-on | cad-off | halt | poweroff | restart | kexec | sw-suspend

*/

#include "v-reboot.h"

int main ( int argc, char **argv )
 {
  int status;

  if ( argc != 2 ) {
usage(); return(1); }

  if ( strcmp(argv[1], "cad-on") == 0 ) 
   { status = reboot(RB_ENABLE_CAD); return status; }
  else if ( strcmp(argv[1], "cad-off") == 0 )
   { status = reboot(RB_DISABLE_CAD); return status; }
  else if ( strcmp(argv[1], "halt") == 0 ) reboot(RB_HALT_SYSTEM);
  else if ( strcmp(argv[1], "poweroff") == 0 ) reboot(RB_POWER_OFF);
  else if ( strcmp(argv[1], "restart") == 0 ) reboot(RB_AUTOBOOT);
  else if ( strcmp(argv[1], "kexec") == 0 ) reboot(RB_KEXEC);
  else if ( strcmp(argv[1], "sw-suspend") == 0 ) reboot(RB_SW_SUSPEND);
  else if ( strcmp(argv[1], "-h") == 0 ) { usage(); return(0); }
  else if ( strcmp(argv[1], "-V") == 0 ) { version(); return(0); }
  else {
#ifdef BIG_CODE
writestr(2, "Wrong argument given!: ");
writestr(2, argv[1]);
writestr(2, "\n");
#endif
usage(); return(1); }

  return 0;
 }

void usage (void)
 {
#ifdef BIG_CODE
  writestr(2, DESC "\n\n");
#endif
#ifndef BIG_CODE
  writestr(2, "v-reboot\n\n");
#endif

  writestr(2, "Usage:\n");
  writestr(2, "\tv-reboot -h | -V | cad-on | cad-off | halt\n");
  writestr(2, "\t | poweroff | restart | kexec | sw-suspend\n");
  writestr(2, "\n\tOnly one argument can be given.\n");
#ifdef BIG_CODE
  writestr(2, "\n\tcad-on\t Ctrl-Alt-Delete causes immediate restart.\n");
  writestr(2, "\tcad-off\t Ctrl-Alt-Delete sends SIGINT to process 1.\n");
  writestr(2, "\thalt\t Halt system immediately.\n");
  writestr(2, "\tpoweroff Power off system immediately.\n");
  writestr(2, "\trestart\t Reboot system immediately.\n");
  writestr(2, "\tkexec\t Restart system using previously loaded Linux Kernel.\n");
  writestr(2, "\trestart\t Suspend system using software suspend.\n");
  writestr(2, "\t-h\t Show help.\n");
  writestr(2, "\t-V\t Show License and Version.\n");

#endif
 }

void version (void)
 {
#ifdef BIG_CODE
  writestr(2, DESC "\n");
#endif
#ifndef BIG_CODE
  writestr(2, "v-reboot\n");
#endif

  writestr(2, "Version: " VERSION "\n");
  writestr(2, "License: " LICENSE "\n");
 }

void writestr ( int fd, char *str )
 {
 size_t len;
 ssize_t bywr = 0, wrstat;
 len = strlen (str);

 do
  {
   wrstat = write (fd, (void *) (str + bywr) , (len - bywr) );
   if ( wrstat == -1 ) exit(1);
   bywr += wrstat;
  }
 while ( bywr != (ssize_t) len );

 fsync(fd);

 }

