// v-rebootp - Execute a Linux reboot syscall

// v-rebootp.c

/*

Copyright Notice:

Copyright (C) 2013 R Veera Kumar

This file is part of v-tools.

v-tools is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

v-tools is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with v-tools.  If not, see <http://www.gnu.org/licenses/>.

See the COPYING file distributed together for license information.

See the AUTHOR file distributed together for authors information.

*/

/*

USAGE: v-rebootp -h | -V | cad-on | cad-off | halt | poweroff | restart | kexec | sw-suspend

*/

#include "v-rebootp.h"

int main ( int argc, char **argv )
 {
  int status;

  if ( argc != 2 ) {
usage(); exit(1); }

  if ( strcmp(argv[1], "cad-on") == 0 ) 
   { status = reboot(RB_ENABLE_CAD); return status; }
  else if ( strcmp(argv[1], "cad-off") == 0 )
   { status = reboot(RB_DISABLE_CAD); return status; }
  else if ( strcmp(argv[1], "halt") == 0 ) reboot(RB_HALT_SYSTEM);
  else if ( strcmp(argv[1], "poweroff") == 0 ) reboot(RB_POWER_OFF);
  else if ( strcmp(argv[1], "restart") == 0 ) reboot(RB_AUTOBOOT);
  else if ( strcmp(argv[1], "kexec") == 0 ) reboot(RB_KEXEC);
  else if ( strcmp(argv[1], "sw-suspend") == 0 ) reboot(RB_SW_SUSPEND);
  else if ( strcmp(argv[1], "-h") == 0 ) { usage(); exit(0); }
  else if ( strcmp(argv[1], "-V") == 0 ) { version(); return(0); }
  else {
#ifdef BIG_CODE
 fprintf(stderr, "Wrong argument given!: %s\n", argv[1]);
#endif
usage(); exit(1); }

  return 0;
 }

void usage (void)
 {
#ifdef BIG_CODE
  fputs(DESC "\n\n", stderr);
#endif
#ifndef BIG_CODE
  fputs("v-rebootp\n\n", stderr);
#endif
  fputs("Usage:\n", stderr);
  fputs("\tv-rebootp -h | -V | cad-on | cad-off | halt\n", stderr);
  fputs("\t | poweroff | restart | kexec | sw-suspend\n", stderr);
  fputs("\n\tOnly one argument can be given.\n", stderr);
#ifdef BIG_CODE
  fputs("\n\tcad-on\t Ctrl-Alt-Delete causes immediate restart.\n", stderr);
  fputs("\tcad-off\t Ctrl-Alt-Delete sends SIGINT to process 1.\n", stderr);
  fputs("\thalt\t Halt system immediately.\n", stderr);
  fputs("\tpoweroff Power off system immediately.\n", stderr);
  fputs("\trestart\t Reboot system immediately.\n", stderr);
  fputs("\tkexec\t Restart system using previously loaded Linux Kernel.\n", stderr);
  fputs("\trestart\t Suspend system using software suspend.\n", stderr);
  fputs("\t-h\t Show help.\n", stderr);
  fputs("\t-V\t Show License and Version.\n", stderr);
#endif
 }

void version (void)
 {
#ifdef BIG_CODE
  fputs(DESC "\n", stderr);
#endif
#ifndef BIG_CODE
  fputs("v-rebootp\n", stderr);
#endif

  fputs("Version: " VERSION "\n", stderr);
  fputs("License: " LICENSE "\n", stderr);
 }

