// v-wrstrp - Write a string to screen or file

// v-wrstrp.c

/*

Copyright Notice:

Copyright (C) 2013 R Veera Kumar

This file is part of v-tools.

v-tools is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

v-tools is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with v-tools.  If not, see <http://www.gnu.org/licenses/>.

See the COPYING file distributed together for license information.

See the AUTHOR file distributed together for authors information.

*/

#include "v-wrstrp.h"

// USAGE: v-wrstrp [ -q ] [ -a ] [ -w ] [ -e ] [ -n ] [ -h ] [ -V ] [ -f FILE ] -s STRING

// Global variables

int q = 0;
int n = 0;

// Global variables

int main ( int argc, char *argv[])
 {
  int a, w, e, f, s, cnt;
  a = e = f = s = 0;
  w = -1;

  extern int n;
  size_t len;

  char *fn, *sv, *esc_strp;
  char *fma = "a", *fmw = "w", *fm;
  FILE *fs = stdout;
  FILE *fp = NULL;
 
  if ( ( argc < 2 ) || ( argc > 11 ) )
   { usage(); exit(1); }

  cnt = 1;
  while ( cnt <= (argc-1) )
   {
    if ( strcmp(argv[cnt], "-e") == 0 )
     { if ( e != 1 ) { e = 1; cnt++; continue; } else { L_OPT_ERR; usage(); exit(1); } }
    
    else if ( strcmp(argv[cnt], "-n") == 0 )
     { if ( n != 1 ) { n = 1; cnt++; continue; } else { L_OPT_ERR; usage(); exit(1); } }

    else if ( strcmp(argv[cnt], "-a") == 0 )
     {
      if ( a != 1 )
       {
        if ( w == 1 ) { L_PARM_ONE_AT_TIME_ERR; exit(1); }
        else { a = 1; w = 0; cnt++; continue;}
       }
      else { L_OPT_ERR; usage(); exit(1); }
     }

    else if ( strcmp(argv[cnt], "-w") == 0 )
     {
      if ( w != 1 )
       {
        if ( a == 1 ) { L_PARM_ONE_AT_TIME_ERR; exit(1); }
        else { w = 1; cnt++; continue; }
       }
      else { L_OPT_ERR; usage(); exit(1); }
     }

    else if ( strcmp(argv[cnt], "-f") == 0 )
     if ( f != 1 )
      { f = 1; if ( (cnt+1) > (argc-1) ) { L_OPT_ARG_ERR; exit(1); }; fn = argv[cnt+1]; cnt +=2; continue; }
     else { L_OPT_ERR; usage(); exit(1); }

    else if ( strcmp(argv[cnt], "-s") == 0 )
     if ( s != 1 )
      { s = 1; if ( (cnt+1) > (argc-1) ) { L_OPT_ARG_ERR; exit(1); };  sv = argv[cnt+1]; cnt +=2; continue; }
     else { L_OPT_ERR; usage(); exit(1); }

    else if ( strcmp(argv[cnt], "-q") == 0 )
     { if ( q != 1 ) { q = 1; cnt++; continue; } else { L_OPT_ERR; usage(); exit(1); } }

    else if ( strcmp(argv[cnt], "-h") == 0 )
     { usage(); exit(0); }

    else if ( strcmp(argv[cnt], "-V") == 0 )
     { version(); exit(0); }

    else { L_PARM_UNKNOWN_ERR(argv[cnt]); usage(); exit(1); }

   }

  if ( ( w == 1 ) || ( w == -1 ) ) fm = fmw;
  else if ( a == 1 ) fm = fma;

  if ( f == 1 )
   {
    if ( (fp = fopen(fn, fm)) == NULL ) { L_FILE_OPEN_ERR; exit(1); }
    fs = fp;
   }

  if ( s != 1 ) { usage(); exit(1); }
  else
   {
    if ( n == 1 )
     {
      if ( e == 1 )
       {
        esc_strp = str_esc_proc(sv, &len);
        if ( fwrite( esc_strp, 1, len, fs) != len ) exit(1);
       }
      else fprintf(fs, "%s", sv);
     }
    else
     {
      if ( e == 1 )
       {
        esc_strp = str_esc_proc(sv, &len);
        if ( fwrite( esc_strp, 1, len, fs) != len ) exit(1);
       }
      else
       {
        fprintf(fs, "%s\n", sv);
       }
     }
   }

  if ( fp != NULL ) fclose(fp);

  return 0;
 }

char * str_esc_proc (char * str, size_t * len)
 {

  char array[4];
  char * mem, * memstp;
  size_t strlgt, cnt;
  int edn, i;

  extern int n;

  strlgt = strlen(str);

// +1 for '\0' and +1 for possible n=0 '\n'
  mem = (char *) malloc(strlgt+1+1);
  memstp = mem;

  if ( mem == NULL ) {  L_ERR_MALLOC_ALLOC; exit(1); }

  memcpy(mem, str, strlgt+1);

  cnt = 0;

  while ( *mem != '\0' )
   {
    if ( *mem++ != '\\' ) { *(memstp + cnt) = *(mem - 1); ++cnt; continue; }
    else
     {
      if ( *mem == '\\' ) *(memstp + cnt) = '\\';
      else if ( *mem == 'a' ) *(memstp + cnt) = '\a';
      else if ( *mem == 'b' ) *(memstp + cnt) = '\b';
      else if ( *mem == 'e' ) *(memstp + cnt) = '\x1b';
      else if ( *mem == 'f' ) *(memstp + cnt) = '\f';
      else if ( *mem == 'n' ) *(memstp + cnt) = '\n';
      else if ( *mem == 'r' ) *(memstp + cnt) = '\r';
      else if ( *mem == 't' ) *(memstp + cnt) = '\t';
      else if ( *mem == 'v' ) *(memstp + cnt) = '\v';
      else if ( *mem == 'c' ) { *(memstp + cnt) = '\0'; return (memstp); }
      else if ( *mem == '0' )
       {
        edn = 0;
        if ( ( *(mem+1) < '0' ) && ( *(mem+1) > '8' ) ) *(memstp + cnt) = '\0';
        else 
         {
          edn = 1;
          if ( ( *(mem+2) >= '0' ) && ( *(mem+2) <= '8' ) ) edn = 2;
          if ( ( *(mem+3) >= '0' ) && ( *(mem+3) <= '8' ) ) edn = 3;

           for ( i = 0; i < edn; ++i)
            {
             array[i] = *(mem + i + 1);
            }
           array[i] = '\0';
           *(memstp + cnt) = (char) strtol(array, NULL, 8);
           mem += i;
         }
       }
      else if ( *mem == 'x' )
       {
        edn = 0;
        if ( !
            (
            ( ( *(mem+1) >= '0' ) && ( *(mem+1) <= '9' ) ) ||
            ( ( *(mem+1) >= 'a' ) && ( *(mem+1) <= 'f' ) ) ||
            ( ( *(mem+1) >= 'A' ) && ( *(mem+1) <= 'F' ) )
            )
           ) { *(memstp + cnt) = '\\'; ++cnt; *(memstp + cnt) = 'x'; }
        else 
         {
          edn = 1;
          if (
              ( ( *(mem+2) >= '0' ) && ( *(mem+2) <= '9' ) ) ||
              ( ( *(mem+2) >= 'a' ) && ( *(mem+2) <= 'f' ) ) ||
              ( ( *(mem+2) >= 'A' ) && ( *(mem+2) <= 'F' ) )
             ) edn = 2;

           for ( i = 0; i < edn; ++i)
            {
             array[i] = *(mem + i + 1);
            }
           array[i] = '\0';
           *(memstp + cnt) = (char) strtol(array, NULL, 16);
           mem += i;
         }
       }

      else ++cnt;

      ++mem;
      ++cnt;
     } 
   }

  if ( n == 0 ) { *(memstp + cnt) = '\n'; ++cnt; }
  *(memstp + cnt) = '\0';

  *len = cnt;

  return (memstp);
 
 }

void usage ( void )
 {
  if ( q != 1)
  {
#ifdef BIG_CODE
   fputs(DESC "\n\n", stderr);
#endif
   fputs("Usage:\n", stderr);
   fputs("\tv-wrstrp [ -a ] [ -w ] [ -e ] [ -n ] [ -h ] [ -f FILE ] -s STRING\n", stderr);
   fputs("\n\tEach argument can be given only once.\n", stderr);
#ifdef BIG_CODE
   fputs("\n\t-a\t Append to FILE\n", stderr);
   fputs("\t-w\t Write to FILE; Truncating if exists.\n", stderr);
   fputs("\t-e\t Works as if not given; currently!\n", stderr);
   fputs("\t-n\t Don't put a newline at end\n", stderr);
   fputs("\t-f\t Write to given FILE; instead of stdout\n", stderr);
   fputs("\t-s\t Write the STRING.\n", stderr);
   fputs("\t-q\t Always give it as first parameter; else cmdline option parser may\n", stderr);
   fputs("\t  \t output error messages. Don't print any messages to screen. Be quiet.\n", stderr);
   fputs("\t-h\t Show usage.\n", stderr);
   fputs("\t-V\t Show License and Version.\n", stderr);
   fputs("\n\tIf -e argument is used then these backslash escapes\n",stderr);
   fputs("\tare recognized:\n",stderr);
   fputs("\n\t\\\\\tbackslash\t\\a\t alert (BEL)\t\\b\tbackspace\n",stderr);
   fputs("\t\\c\tproduce no further output; no trailing newline\n",stderr);
   fputs("\t\\e\tescape\t\t\\f\tform feed\t\\n\tnew line\n",stderr);
   fputs("\t\\r\tcarriage return\t\\t\thorizontal tab\t\\v\tvertical tab\n",stderr);
   fputs("\t\\0NNN\tbyte with octal value NNN (0-3 digits)\n",stderr);
   fputs("\t\\xHH\tbyte with hexadecimal value HH (1-2 digits)\n",stderr);
#endif

  }
 }

void version (void)
 {
#ifdef BIG_CODE
  fputs(DESC "\n", stderr);
#endif
#ifndef BIG_CODE
  fputs("v-wrstrp\n", stderr);
#endif

  fputs("Version: " VERSION "\n", stderr);
  fputs("License: " LICENSE "\n", stderr);
 }

#ifdef BIG_CODE
void opt_err (void)
 {
  if ( q != 1 )
  fputs("Each argument can be given only once!\n", stderr);
 }

void opt_arg_err (void)
 {
  if ( q != 1 )
  fputs("Neccessary argument for specified option not given!\n", stderr);
 }

void file_open_err (void)
 {
  if ( q != 1 )
  fputs("Could not open file for writing!\n", stderr);
 }

void parm_one_at_time_err (void)
 {
  if ( q != 1 )
  fputs("Either -a or -w can be given; not both!\n", stderr);
 }

void parm_unknown_err (char *arg)
 {
  if ( q != 1 )
  fprintf(stderr, "Unknown argument given!: %s\n", arg);
 }


void err_malloc_alloc (void)
 {
  if ( q != 1 )
  fputs("Couldn't allocate memory!\n", stderr);
 }

#endif

